from LibHanger.Library.uwGlobals import globalValues
from parroter.Library.kojima.kojimaConfig import kojimaConfig

class parroterGlobal(globalValues):
    
    """
    parroterグローバル設定クラス
    """
    
    def __init__(self):
        
        """
        コンストラクタ
        """
        
        # 基底側コンストラクタ呼び出し
        super().__init__()
        